use std::str::FromStr;

use crate::*;

#[derive(Debug, Clone)]
pub struct Enigma {
    rotors: Vec<Rotor>,
    plugboard: Plugboard,
    reflector: Reflector,
}

impl Enigma {
    pub fn new(rotors: Vec<Rotor>, plugboard: Plugboard, reflector: Reflector) -> Self {
        Enigma {
            rotors,
            plugboard,
            reflector,
        }
    }
    pub fn encode(&mut self, input: &str) -> Result<String, &'static str> {
        let output: Result<String, &'static str> = input
            .chars()
            .map(|c| {
                self.rotate_rotors();

                let mut letter = Letter::from_str(&c.to_string())?;
                letter = self.plugboard.letter(letter);

                for rotor in self.rotors.iter() {
                    letter = rotor.forwards(letter);
                }

                letter = self.reflector.letter(letter);

                for rotor in self.rotors.iter().rev() {
                    letter = rotor.backwards(letter);
                }

                letter = self.plugboard.letter(letter);

                Ok(char::from(letter))
            })
            .collect();

        output
    }

    fn rotate_rotors(&mut self) {
        let mut rotate = true;

        for rotor in self.rotors.iter_mut() {
            let rotate_next = rotor.at_notch();

            if rotate {
                rotor.rotate();
            }

            rotate = rotate_next;
        }
    }
}

impl Random for Enigma {
    fn random() -> Self {
        todo!()
    }
}
